@@ -64,7 +64,7 @@ class TestMultiDbClient:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
@@ -486,7 +486,9 @@ class TestMultiDbClient:
                 ):
                     # With skip_unhealthy=False, should raise exception
                     with pytest.raises(UnhealthyDatabaseException):
-                        await client.add_database(new_db_config, skip_unhealthy=False)
+                        await client.add_database(
+                            new_db_config, skip_initial_health_check=False
+                        )
 
                 # Database list should remain unchanged
                 assert len(client.get_databases()) == 2
@@ -789,7 +791,7 @@ class TestInitialHealthCheckPolicy:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
@@ -825,7 +827,7 @@ class TestInitialHealthCheckPolicy:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
@@ -863,7 +865,7 @@ class TestInitialHealthCheckPolicy:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {"initial_health_check_policy": InitialHealthCheck.ANY_HEALTHY},
+                {"initial_health_check_policy": InitialHealthCheck.ONE_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
@@ -898,7 +900,7 @@ class TestInitialHealthCheckPolicy:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {"initial_health_check_policy": InitialHealthCheck.ANY_HEALTHY},
+                {"initial_health_check_policy": InitialHealthCheck.ONE_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
