@@ -69,7 +69,7 @@ class TestPipeline:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
@@ -325,7 +325,7 @@ class TestTransaction:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_HEALTHY},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
