@@ -4,10 +4,10 @@ import string
 import threading
 from time import sleep
 from typing import Optional, Tuple, Union
+from unittest.mock import Mock, call
 
 import pytest
 import redis
-from mock.mock import Mock, call
 from redis import AuthenticationError, DataError, Redis, ResponseError
 from redis.auth.err import RequestTokenErr
 from redis.backoff import NoBackoff
@@ -319,7 +319,13 @@ class TestStreamingCredentialProvider:
     def test_re_auth_all_connections(self, credential_provider):
         mock_connection = Mock(spec=ConnectionInterface)
         mock_connection.retry = Retry(NoBackoff(), 0)
+        mock_connection.host = "localhost"
+        mock_connection.port = 6379
+        mock_connection.db = 0
         mock_another_connection = Mock(spec=ConnectionInterface)
+        mock_another_connection.host = "localhost"
+        mock_another_connection.port = 6379
+        mock_another_connection.db = 0
         mock_pool = Mock(spec=ConnectionPool)
         mock_pool.connection_kwargs = {
             "credential_provider": credential_provider,
@@ -368,13 +374,22 @@ class TestStreamingCredentialProvider:
     def test_re_auth_partial_connections(self, credential_provider):
         mock_connection = Mock(spec=ConnectionInterface)
         mock_connection.retry = Retry(NoBackoff(), 3)
+        mock_connection.host = "localhost"
+        mock_connection.port = 6379
+        mock_connection.db = 0
         mock_another_connection = Mock(spec=ConnectionInterface)
         mock_another_connection.retry = Retry(NoBackoff(), 3)
+        mock_another_connection.host = "localhost"
+        mock_another_connection.port = 6379
+        mock_another_connection.db = 0
         mock_failed_connection = Mock(spec=ConnectionInterface)
         mock_failed_connection.read_response.side_effect = ConnectionError(
             "Failed auth"
         )
         mock_failed_connection.retry = Retry(NoBackoff(), 3)
+        mock_failed_connection.host = "localhost"
+        mock_failed_connection.port = 6379
+        mock_failed_connection.db = 0
         mock_pool = Mock(spec=ConnectionPool)
         mock_pool.connection_kwargs = {
             "credential_provider": credential_provider,
@@ -434,8 +449,14 @@ class TestStreamingCredentialProvider:
         mock_pubsub_connection.should_reconnect = Mock(return_value=False)
         mock_pubsub_connection.credential_provider = credential_provider
         mock_pubsub_connection.retry = Retry(NoBackoff(), 3)
+        mock_pubsub_connection.host = "localhost"
+        mock_pubsub_connection.port = 6379
+        mock_pubsub_connection.db = 0
         mock_another_connection = Mock(spec=ConnectionInterface)
         mock_another_connection.retry = Retry(NoBackoff(), 3)
+        mock_another_connection.host = "localhost"
+        mock_another_connection.port = 6379
+        mock_another_connection.db = 0
 
         mock_pool = Mock(spec=ConnectionPool)
         mock_pool.connection_kwargs = {
@@ -495,8 +516,14 @@ class TestStreamingCredentialProvider:
         mock_pubsub_connection.should_reconnect = Mock(return_value=False)
         mock_pubsub_connection.credential_provider = credential_provider
         mock_pubsub_connection.retry = Retry(NoBackoff(), 3)
+        mock_pubsub_connection.host = "localhost"
+        mock_pubsub_connection.port = 6379
+        mock_pubsub_connection.db = 0
         mock_another_connection = Mock(spec=ConnectionInterface)
         mock_another_connection.retry = Retry(NoBackoff(), 3)
+        mock_another_connection.host = "localhost"
+        mock_another_connection.port = 6379
+        mock_another_connection.db = 0
 
         mock_pool = Mock(spec=ConnectionPool)
         mock_pool.connection_kwargs = {
@@ -558,7 +585,13 @@ class TestStreamingCredentialProvider:
         ]
         mock_connection = Mock(spec=ConnectionInterface)
         mock_connection.retry = Retry(NoBackoff(), 0)
+        mock_connection.host = "localhost"
+        mock_connection.port = 6379
+        mock_connection.db = 0
         mock_another_connection = Mock(spec=ConnectionInterface)
+        mock_another_connection.host = "localhost"
+        mock_another_connection.port = 6379
+        mock_another_connection.db = 0
         mock_pool = Mock(spec=ConnectionPool)
         mock_pool.connection_kwargs = {
             "credential_provider": credential_provider,
