@@ -7,9 +7,14 @@ import pytest
 
 from redis.event import EventDispatcher, OnCommandsFailEvent
 from redis.multidb.circuit import State as CBState, PBCircuitBreakerAdapter
+from redis.multidb.config import InitialHealthCheck, DatabaseConfig
 from redis.multidb.database import SyncDatabase
 from redis.multidb.client import MultiDBClient
-from redis.multidb.exception import NoValidDatabaseException
+from redis.multidb.exception import (
+    NoValidDatabaseException,
+    InitialHealthCheckFailedError,
+    UnhealthyDatabaseException,
+)
 from redis.multidb.failover import WeightBasedFailoverStrategy
 from redis.multidb.failure_detector import FailureDetector
 from redis.multidb.healthcheck import HealthCheck
@@ -60,7 +65,7 @@ class TestMultiDbClient:
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
             (
-                {},
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
                 {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
                 {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
@@ -396,75 +401,6 @@ class TestMultiDbClient:
             finally:
                 client.close()
 
-    @pytest.mark.parametrize(
-        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
-        [
-            (
-                {},
-                {"weight": 0.2, "circuit": {"state": CBState.OPEN}},
-                {"weight": 0.7, "circuit": {"state": CBState.OPEN}},
-                {"weight": 0.5, "circuit": {"state": CBState.OPEN}},
-            ),
-        ],
-        indirect=True,
-    )
-    def test_execute_command_throws_exception_on_failed_initialization(
-        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
-    ):
-        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
-        mock_multi_db_config.health_checks = [mock_hc]
-
-        with patch.object(mock_multi_db_config, "databases", return_value=databases):
-            mock_hc.check_health.return_value = False
-
-            client = MultiDBClient(mock_multi_db_config)
-            try:
-                assert (
-                    mock_multi_db_config.failover_strategy.set_databases.call_count == 1
-                )
-
-                with pytest.raises(
-                    NoValidDatabaseException,
-                    match="Initial connection failed - no active database found",
-                ):
-                    client.set("key", "value")
-            finally:
-                client.close()
-
-    @pytest.mark.parametrize(
-        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
-        [
-            (
-                {},
-                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
-                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
-                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
-            ),
-        ],
-        indirect=True,
-    )
-    def test_add_database_throws_exception_on_same_database(
-        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
-    ):
-        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
-        mock_multi_db_config.health_checks = [mock_hc]
-
-        with patch.object(mock_multi_db_config, "databases", return_value=databases):
-            mock_hc.check_health.return_value = False
-
-            client = MultiDBClient(mock_multi_db_config)
-            try:
-                assert (
-                    mock_multi_db_config.failover_strategy.set_databases.call_count == 1
-                )
-
-                with pytest.raises(ValueError, match="Given database already exists"):
-                    client.add_database(mock_db)
-                    assert len(mock_hc.check_health.call_args_list) == 3
-
-            finally:
-                client.close()
-
     @pytest.mark.parametrize(
         "mock_multi_db_config,mock_db, mock_db1, mock_db2",
         [
@@ -480,13 +416,22 @@ class TestMultiDbClient:
     def test_add_database_makes_new_database_active(
         self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
     ):
+        """
+        Test that add_database with a DatabaseConfig creates a new database
+        and makes it active if it has the highest weight.
+        """
+
         databases = create_weighted_list(mock_db, mock_db2)
         mock_multi_db_config.health_checks = [mock_hc]
 
-        with patch.object(mock_multi_db_config, "databases", return_value=databases):
-            mock_db1.client.execute_command.return_value = "OK1"
-            mock_db2.client.execute_command.return_value = "OK2"
+        # Create a DatabaseConfig for the new database with highest weight
+        new_db_config = DatabaseConfig(
+            weight=0.8,  # Higher than mock_db2's 0.5
+            from_url="redis://localhost:6379",
+        )
 
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db2.client.execute_command.return_value = "OK2"
             mock_hc.check_health.return_value = True
 
             client = MultiDBClient(mock_multi_db_config)
@@ -495,13 +440,91 @@ class TestMultiDbClient:
                     mock_multi_db_config.failover_strategy.set_databases.call_count == 1
                 )
 
+                # Initially mock_db2 is active (highest weight among initial databases)
                 assert client.set("key", "value") == "OK2"
-                assert len(mock_hc.check_health.call_args_list) == 6
+                initial_hc_count = len(mock_hc.check_health.call_args_list)
 
-                client.add_database(mock_db1)
-                assert len(mock_hc.check_health.call_args_list) == 9
+                # Mock the client class to return a mock client for the new database
+                mock_new_client = Mock()
+                mock_new_client.execute_command.return_value = "OK_NEW"
+                mock_new_client.connection_pool = Mock()
 
-                assert client.set("key", "value") == "OK1"
+                with patch.object(
+                    mock_multi_db_config.client_class,
+                    "from_url",
+                    return_value=mock_new_client,
+                ):
+                    client.add_database(new_db_config)
+
+                # Health check should have been called for the new database
+                assert len(mock_hc.check_health.call_args_list) > initial_hc_count
+
+                # New database should be active since it has highest weight
+                assert client.set("key", "value") == "OK_NEW"
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_add_database_skip_unhealthy_false_raises_exception(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that add_database with skip_unhealthy=False raises an exception
+        when the new database fails health check due to an exception.
+        """
+        databases = create_weighted_list(mock_db, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        new_db_config = DatabaseConfig(
+            weight=0.8,
+            from_url="redis://localhost:6379",
+        )
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db2.client.execute_command.return_value = "OK2"
+
+            # Health check returns True for existing databases, raises exception for new one
+            def mock_check_health(database):
+                if database in [mock_db, mock_db2]:
+                    return True
+                # Raise an exception for the new database to trigger UnhealthyDatabaseException
+                raise ConnectionError("Connection refused")
+
+            mock_hc.check_health.side_effect = mock_check_health
+
+            client = MultiDBClient(mock_multi_db_config)
+            try:
+                # Initially mock_db2 is active
+                assert client.set("key", "value") == "OK2"
+
+                mock_new_client = Mock()
+                mock_new_client.execute_command.return_value = "OK_NEW"
+                mock_new_client.connection_pool = Mock()
+
+                with patch.object(
+                    mock_multi_db_config.client_class,
+                    "from_url",
+                    return_value=mock_new_client,
+                ):
+                    # With skip_unhealthy=False, should raise exception
+                    with pytest.raises(UnhealthyDatabaseException):
+                        client.add_database(
+                            new_db_config, skip_initial_health_check=False
+                        )
+
+                # Database list should remain unchanged
+                assert len(client.get_databases()) == 2
             finally:
                 client.close()
 
@@ -733,3 +756,386 @@ class TestMultiDbClient:
                     client.set_active_database(mock_db1)
             finally:
                 client.close()
+
+
+@pytest.mark.onlynoncluster
+class TestGeoFailoverMetricRecording:
+    """Tests for geo failover metric recording in MultiDBClient."""
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_manual_failover_records_metric(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that set_active_database records geo failover metric with MANUAL reason.
+        """
+        from redis.observability.attributes import GeoFailoverReason
+
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db1.client.execute_command.return_value = "OK1"
+            mock_db.client.execute_command.return_value = "OK"
+            mock_hc.check_health.return_value = True
+
+            client = MultiDBClient(mock_multi_db_config)
+            try:
+                # Initial active database should be mock_db1 (highest weight)
+                assert client.set("key", "value") == "OK1"
+
+                # Now manually switch to mock_db
+                # Patch at the module where it's imported
+                with patch(
+                    "redis.multidb.command_executor.record_geo_failover"
+                ) as mock_record_geo_failover:
+                    client.set_active_database(mock_db)
+
+                    # Verify record_geo_failover was called with correct arguments
+                    mock_record_geo_failover.assert_called_once()
+                    call_kwargs = mock_record_geo_failover.call_args[1]
+                    assert call_kwargs["fail_from"] == mock_db1
+                    assert call_kwargs["fail_to"] == mock_db
+                    assert call_kwargs["reason"] == GeoFailoverReason.MANUAL
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_automatic_failover_records_metric(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that automatic failover records geo failover metric with AUTOMATIC reason.
+        """
+        from redis.observability.attributes import GeoFailoverReason
+
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db1.client.execute_command.return_value = "OK1"
+            mock_db2.client.execute_command.return_value = "OK2"
+            mock_hc.check_health.return_value = True
+
+            client = MultiDBClient(mock_multi_db_config)
+            try:
+                # Initial active database should be mock_db1 (highest weight)
+                assert client.set("key", "value") == "OK1"
+
+                # Simulate mock_db1 becoming unhealthy (circuit open)
+                mock_db1.circuit.state = CBState.OPEN
+
+                # Configure the failover strategy to return mock_db2 when database() is called
+                # The DefaultFailoverStrategyExecutor calls self._strategy.database()
+                mock_multi_db_config.failover_strategy.database.return_value = mock_db2
+
+                # Patch at the module where it's imported
+                with patch(
+                    "redis.multidb.command_executor.record_geo_failover"
+                ) as mock_record_geo_failover:
+                    # Execute a command - this should trigger automatic failover
+                    assert client.set("key", "value") == "OK2"
+
+                    # Verify record_geo_failover was called with AUTOMATIC reason
+                    mock_record_geo_failover.assert_called_once()
+                    call_kwargs = mock_record_geo_failover.call_args[1]
+                    assert call_kwargs["fail_from"] == mock_db1
+                    assert call_kwargs["fail_to"] == mock_db2
+                    assert call_kwargs["reason"] == GeoFailoverReason.AUTOMATIC
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_no_metric_recorded_when_same_database(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that no geo failover metric is recorded when active database doesn't change.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db1.client.execute_command.return_value = "OK1"
+            mock_hc.check_health.return_value = True
+
+            client = MultiDBClient(mock_multi_db_config)
+            try:
+                # Initial active database should be mock_db1 (highest weight)
+                assert client.set("key", "value") == "OK1"
+
+                # Patch at the module where it's imported
+                with patch(
+                    "redis.multidb.command_executor.record_geo_failover"
+                ) as mock_record_geo_failover:
+                    # Set active database to the same database
+                    client.set_active_database(mock_db1)
+
+                    # Verify record_geo_failover was NOT called
+                    mock_record_geo_failover.assert_not_called()
+            finally:
+                client.close()
+
+
+@pytest.mark.onlynoncluster
+class TestInitialHealthCheckPolicy:
+    """Tests for initial health check policy evaluation."""
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_all_healthy_policy_succeeds_when_all_databases_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ALL_HEALTHY policy succeeds when all databases pass health check.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db1.client.execute_command.return_value = "OK1"
+            mock_hc.check_health.return_value = True
+
+            client = MultiDBClient(mock_multi_db_config)
+
+            try:
+                # Should succeed without raising InitialHealthCheckFailedError
+                assert client.set("key", "value") == "OK1"
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.ALL_AVAILABLE},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_all_healthy_policy_fails_when_one_database_unhealthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ALL_HEALTHY policy fails when any database fails health check.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+
+            def mock_check_health(database):
+                # mock_db2 is unhealthy
+                return database != mock_db2
+
+            mock_hc.check_health.side_effect = mock_check_health
+
+            client = MultiDBClient(mock_multi_db_config)
+
+            try:
+                with pytest.raises(
+                    InitialHealthCheckFailedError,
+                    match="Initial health check failed",
+                ):
+                    client.set("key", "value")
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_majority_healthy_policy_succeeds_when_majority_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that MAJORITY_HEALTHY policy succeeds when more than half of databases are healthy.
+        With 3 databases, 2 healthy is a majority.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db1.client.execute_command.return_value = "OK1"
+
+            def mock_check_health(database):
+                # mock_db2 is unhealthy, but 2 out of 3 are healthy (majority)
+                return database != mock_db2
+
+            mock_hc.check_health.side_effect = mock_check_health
+
+            client = MultiDBClient(mock_multi_db_config)
+
+            try:
+                # Should succeed - 2 out of 3 healthy is a majority
+                assert client.set("key", "value") == "OK1"
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.MAJORITY_AVAILABLE},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_majority_healthy_policy_fails_when_minority_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that MAJORITY_HEALTHY policy fails when less than half of databases are healthy.
+        With 3 databases, only 1 healthy is not a majority.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+
+            def mock_check_health(database):
+                # Only mock_db is healthy (1 out of 3 is not a majority)
+                return database == mock_db
+
+            mock_hc.check_health.side_effect = mock_check_health
+
+            client = MultiDBClient(mock_multi_db_config)
+
+            try:
+                with pytest.raises(
+                    InitialHealthCheckFailedError,
+                    match="Initial health check failed",
+                ):
+                    client.set("key", "value")
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.ONE_AVAILABLE},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_any_healthy_policy_succeeds_when_one_database_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ANY_HEALTHY policy succeeds when at least one database is healthy.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_db.client.execute_command.return_value = "OK"
+
+            def mock_check_health(database):
+                # Only mock_db is healthy
+                return database == mock_db
+
+            mock_hc.check_health.side_effect = mock_check_health
+
+            client = MultiDBClient(mock_multi_db_config)
+
+            try:
+                # Should succeed - at least one database is healthy
+                assert client.set("key", "value") == "OK"
+            finally:
+                client.close()
+
+    @pytest.mark.parametrize(
+        "mock_multi_db_config,mock_db, mock_db1, mock_db2",
+        [
+            (
+                {"initial_health_check_policy": InitialHealthCheck.ONE_AVAILABLE},
+                {"weight": 0.2, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.7, "circuit": {"state": CBState.CLOSED}},
+                {"weight": 0.5, "circuit": {"state": CBState.CLOSED}},
+            ),
+        ],
+        indirect=True,
+    )
+    def test_any_healthy_policy_fails_when_no_database_healthy(
+        self, mock_multi_db_config, mock_db, mock_db1, mock_db2, mock_hc
+    ):
+        """
+        Test that ANY_HEALTHY policy fails when no database is healthy.
+        """
+        databases = create_weighted_list(mock_db, mock_db1, mock_db2)
+        mock_multi_db_config.health_checks = [mock_hc]
+
+        with patch.object(mock_multi_db_config, "databases", return_value=databases):
+            mock_hc.check_health.return_value = False
+
+            client = MultiDBClient(mock_multi_db_config)
+
+            try:
+                with pytest.raises(
+                    InitialHealthCheckFailedError,
+                    match="Initial health check failed",
+                ):
+                    client.set("key", "value")
+            finally:
+                client.close()
